@@ -469,11 +469,12 @@ E boolean FDECL(hurtle_step, (genericptr_t, int, int));
 E int FDECL(def_char_to_objclass, (CHAR_P));
 E int FDECL(def_char_to_monclass, (CHAR_P));
 #if !defined(MAKEDEFS_C) && !defined(LEV_LEX_C)
-E void FDECL(assign_graphics, (uchar *,int,int,int));
+E void FDECL(assign_graphics, (glyph_t *,int,int,int));
 E void FDECL(switch_graphics, (int));
 #ifdef REINCARNATION
 E void FDECL(assign_rogue_graphics, (BOOLEAN_P));
 #endif
+E void FDECL(assign_utf8graphics_symbol, (int, glyph_t));
 
 /* ### dungeon.c ### */
 
@@ -485,17 +486,25 @@ E void FDECL(dump_header_html, (const char *));
 E void FDECL(dump_html, (const char *, const char *));
 E void NDECL(dump_init);
 E void NDECL(dump_exit);
-E void FDECL(dump_object, (const char, const char *));
+E void FDECL(dump_object, (const char, const struct obj *, const char *));
 E void FDECL(dump_title, (char *));
 E void FDECL(dump_subtitle, (const char *));
 E void FDECL(dump_line, (const char *, const char *));
 E void NDECL(dump_list_start);
 E void FDECL(dump_list_item, (const char *));
+E void FDECL(dump_list_item_object, (struct obj *));
+E void FDECL(dump_list_item_link, (const char *, const char *));
 E void NDECL(dump_list_end);
+E void NDECL(dump_definition_list_start);
+E void FDECL(dump_definition_list_dd, (const char *));
+E void FDECL(dump_definition_list_dt, (const char *));
+E void NDECL(dump_definition_list_end);
 E void FDECL(dump_containerconts, (struct obj *,BOOLEAN_P,BOOLEAN_P,BOOLEAN_P));
 E char* FDECL(html_escape_character, (const char));
+E char* FDECL(html_link, (const char *, const char *));
 #ifdef DUMP_LOG
 E int NDECL(dump_screenshot);
+E int NDECL(dumpoverview);
 #endif
 
 /* ### dungeon.c ### */
@@ -558,6 +567,7 @@ E void FDECL(init_mapseen, (d_level *));
 E void NDECL(recalc_mapseen);
 E void FDECL(recbranch_mapseen, (d_level *, d_level *));
 E void FDECL(remdun_mapseen, (int));
+E char *FDECL(get_annotation, (d_level *));
 
 /* ### eat.c ### */
 
@@ -661,7 +671,9 @@ E void NDECL(makerogueghost);
 E char *FDECL(fname_encode, (const char *, CHAR_P, char *, char *, int));
 E char *FDECL(fname_decode, (CHAR_P, char *, char *, int));
 E const char *FDECL(fqname, (const char *, int, int));
+#ifndef FILE_AREAS
 E FILE *FDECL(fopen_datafile, (const char *,const char *,int));
+#endif
 E boolean FDECL(uptodate, (int,const char *));
 E void FDECL(store_version, (int));
 #ifdef MFLOPPY
@@ -693,8 +705,12 @@ E int NDECL(delete_savefile);
 E int NDECL(restore_saved_game);
 E void FDECL(compress, (const char *));
 E void FDECL(uncompress, (const char *));
+E void FDECL(compress_area, (const char *, const char *));
+E void FDECL(uncompress_area, (const char *, const char *));
+#ifndef FILE_AREAS
 E boolean FDECL(lock_file, (const char *,int,int));
 E void FDECL(unlock_file, (const char *));
+#endif
 #ifdef USER_SOUNDS
 E boolean FDECL(can_read_file, (const char *));
 #endif
@@ -716,6 +732,7 @@ E void NDECL(really_close);
 #ifdef WHEREIS_FILE
 E void NDECL(touch_whereis);
 E void NDECL(delete_whereis);
+E void FDECL(signal_whereis, (int));
 #endif
 
 /* ### fountain.c ### */
@@ -816,6 +833,7 @@ E boolean NDECL(towelday);
 E int NDECL(night);
 E int NDECL(midnight);
 E boolean NDECL(piday);
+E boolean NDECL(aprilfoolsday);
 E char *FDECL(iso8601, (time_t));
 E char *FDECL(iso8601_duration, (long));
 E char *FDECL(get_formatted_time, (time_t, const char *));
@@ -885,11 +903,13 @@ E void NDECL(free_invbuf);
 E void NDECL(reassign);
 E int NDECL(doorganize);
 E int FDECL(count_objects, (struct obj *));
+E int FDECL(count_unidentified, (struct obj *));
 E int FDECL(count_unpaid, (struct obj *));
 E int FDECL(count_buc, (struct obj *,int));
 E void FDECL(carry_obj_effects, (struct obj *));
 E const char *FDECL(currency, (long));
 E void FDECL(silly_thing, (const char *,struct obj *));
+E struct obj *NDECL(getnextgetobj);
 
 /* ### ioctl.c ### */
 
@@ -1021,7 +1041,7 @@ E int NDECL(max_monster_difficulty);
 
 /* ### mapglyph.c ### */
 
-E void FDECL(mapglyph, (int, int *, int *, unsigned *, int, int));
+E void FDECL(mapglyph, (int, glyph_t *, int *, unsigned *, int, int));
 
 /* ### mcastu.c ### */
 
@@ -1439,6 +1459,7 @@ E void NDECL(objects_init);
 
 E char *FDECL(obj_typename, (int));
 E char *FDECL(simple_typename, (int));
+E char *FDECL(dump_typename, (int));
 E boolean FDECL(obj_is_pname, (struct obj *));
 E char *FDECL(distant_name, (struct obj *,char *(*)(OBJ_P)));
 E char *FDECL(fruitname, (BOOLEAN_P));
@@ -1450,6 +1471,9 @@ E char *FDECL(doname_with_price, (struct obj *));
 E boolean FDECL(not_fully_identified, (struct obj *));
 E char *FDECL(corpse_xname, (struct obj *,BOOLEAN_P));
 E char *FDECL(cxname, (struct obj *));
+#ifdef SORTLOOT
+E char *FDECL(cxname2, (struct obj *));
+#endif
 E char *FDECL(killer_xname, (struct obj *));
 E const char *FDECL(singular, (struct obj *,char *(*)(OBJ_P)));
 E char *FDECL(an, (const char *));
@@ -1477,6 +1501,8 @@ E const char *FDECL(mimic_obj_name, (struct monst *));
 E boolean FDECL(match_optname, (const char *,const char *,int,BOOLEAN_P));
 E void NDECL(initoptions);
 E void FDECL(parseoptions, (char *,BOOLEAN_P,BOOLEAN_P));
+E boolean FDECL(parse_monster_color, (char *));
+E boolean FDECL(parse_symbol, (const char *));
 E int NDECL(doset);
 E int NDECL(dotogglepickup);
 E void NDECL(option_help);
@@ -1508,6 +1534,7 @@ E int NDECL(dowhatdoes);
 E char *FDECL(dowhatdoes_core,(CHAR_P, char *));
 E int NDECL(dohelp);
 E int NDECL(dohistory);
+E void FDECL(checkfile, (char *,struct permonst *,BOOLEAN_P,BOOLEAN_P));
 
 /* ### pcmain.c ### */
 
@@ -1593,6 +1620,8 @@ E boolean FDECL(is_autopickup_exception, (struct obj *, BOOLEAN_P));
 
 /* ### pline.c ### */
 
+E void FDECL(msgpline_add, (int, char *));
+E void NDECL(msgpline_free);
 E void VDECL(pline, (const char *,...)) PRINTF_F(1,2);
 E void VDECL(Norep, (const char *,...)) PRINTF_F(1,2);
 E void NDECL(free_youbuf);
@@ -1780,8 +1809,8 @@ E NhRegion *FDECL(visible_region_at, (XCHAR_P,XCHAR_P));
 E void FDECL(show_region, (NhRegion*, XCHAR_P, XCHAR_P));
 E void FDECL(save_regions, (int,int));
 E void FDECL(rest_regions, (int,BOOLEAN_P));
-E NhRegion* FDECL(create_gas_cloud, (XCHAR_P, XCHAR_P, int, size_t));
-E NhRegion* FDECL(create_cthulhu_death_cloud, (XCHAR_P, XCHAR_P, int, size_t));
+E NhRegion* FDECL(create_gas_cloud, (XCHAR_P, XCHAR_P, int, size_t, int));
+E NhRegion* FDECL(create_cthulhu_death_cloud, (XCHAR_P, XCHAR_P, int, size_t, int));
 
 /* ### restore.c ### */
 
@@ -2007,6 +2036,7 @@ E void NDECL(losespells);
 E int NDECL(dovspell);
 E void FDECL(initialspell, (struct obj *));
 E void NDECL(dump_spells);
+E boolean FDECL(parse_spellorder, (char *));
 
 /* ### steal.c ### */
 
@@ -2217,6 +2247,19 @@ E int NDECL(dosh);
 # if defined(SHELL) || defined(DEF_PAGER) || defined(DEF_MAILREADER)
 E int FDECL(child, (int));
 # endif
+#ifdef FILE_AREAS
+E char *FDECL(make_file_name, (const char *, const char *));
+E FILE *FDECL(fopen_datafile_area, (const char *,const char *,const char *,
+				BOOLEAN_P));
+E FILE *FDECL(freopen_area, (const char *,const char *,const char *, FILE *));
+E int FDECL(chmod_area, (const char *, const char *, int));
+E int FDECL(open_area, (const char *, const char *, int, int));
+E int FDECL(creat_area, (const char *, const char *, int));
+E boolean FDECL(lock_file_area, (const char *, const char *,int));
+E void FDECL(unlock_file_area, (const char *, const char *));
+E int FDECL(rename_area, (const char *, const char *, const char *));
+E int FDECL(remove_area, (const char *, const char *));
+#endif
 #endif /* UNIX */
 
 /* ### unixres.c ### */
@@ -2454,6 +2497,8 @@ E void FDECL(remove_worm, (struct monst *));
 E void FDECL(place_worm_tail_randomly, (struct monst *,XCHAR_P,XCHAR_P));
 E int FDECL(count_wsegs, (struct monst *));
 E boolean FDECL(worm_known, (struct monst *));
+E void FDECL(flip_worm_segs_vertical, (struct monst *, int));
+E void FDECL(flip_worm_segs_horizontal, (struct monst *, int));
 
 /* ### worn.c ### */
 
@@ -2543,6 +2588,11 @@ E void NDECL(tutorial_redisplay_message);
 E void FDECL(check_tutorial_farlook, (int, int));
 E void FDECL(check_tutorial_command, (char));
 E int FDECL(check_tutorial_location, (int, int, BOOLEAN_P));
+E int FDECL(check_tutorial_oclass, (int));
+
+/* ### unicode.c ### */
+E glyph_t FDECL(get_unicode_codepoint, (int));
+E int FDECL(pututf8char, (glyph_t));
 
 #undef E
 
