@@ -641,6 +641,7 @@ char *buf;
 	else if (IS_DOOR(maploc->typ)) what = "a door";
 	else if (IS_TREE(maploc->typ)) what = "a tree";
 	else if (IS_STWALL(maploc->typ)) what = "a wall";
+	else if (IS_DEADTREE(maploc->typ)) what = "a dead tree";
 	else if (IS_ROCK(maploc->typ)) what = "a rock";
 	else if (IS_THRONE(maploc->typ)) what = "a throne";
 	else if (IS_FOUNTAIN(maploc->typ)) what = "a fountain";
@@ -978,6 +979,27 @@ dokick()
 		    }
 		    goto ouch;
 		}
+		if(IS_DEADTREE(maploc->typ)) {
+		    if(Levitation) goto dumb;
+		    You("kick %s.", Blind ? something : "the dead tree");
+		    switch (rn2(4)) {
+			case 0:	goto ouch;
+			case 1:	pline("The tree is tottering...");
+				break;
+			case 2:	pline("Some branches are swinging...");
+				break;
+			case 3:	if (!may_dig(x,y)) goto ouch;
+				pline("The dead tree falls down.");
+				maploc->typ = ROOM;
+				if (Blind)
+				    feel_location(x,y);	/* we know it's gone */
+				else
+				    newsym(x,y);
+				unblock_point(x,y);	/* vision */
+				break;
+		    }
+		    return(1);
+		}
 #ifdef SINKS
 		if(IS_SINK(maploc->typ)) {
 		    int gend = poly_gender();
