@@ -1458,12 +1458,13 @@ eatcorpse(otmp)		/* called when a corpse is selected as food */
 		  ? "is delicious" : "tastes terrible");
 	}
 
-	/* Eating slimy or oily corpses make your fingers slippery.
+	/* Eating slimy or oily corpses makes your fingers slippery.
 	   Note: Snakes are not slimy. */
 	if ((amorphous(&mons[mnum]) || slithy(&mons[mnum]) ||
 	     mons[mnum].mlet == S_BLOB) &&
 	    mons[mnum].mlet != S_SNAKE && mons[mnum].mlet != S_NAGA &&
-	    mons[mnum].mlet != S_MIMIC && rnf(1,5)) {
+	    mons[mnum].mlet != S_MIMIC && rnf(1,5) &&
+	    !is_vampiric(youmonst.data)) {
 		pline("Eating this %s corpse makes your %s %s slippery.",
 		      amorphous(&mons[mnum]) ? "glibbery" : "slimy",
 		      makeplural(body_part(FINGER)),
@@ -2241,7 +2242,7 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 	    victual.reqtime = objects[otmp->otyp].oc_delay;
 	    if (otmp->otyp != FORTUNE_COOKIE &&
 		(otmp->cursed ||
-		 (((monstermoves - otmp->age) > (int) otmp->blessed ? 50:30) &&
+		 (((monstermoves - otmp->age) > (otmp->blessed ? 50 : 30)) &&
 		(otmp->orotten || !rn2(7))))) {
 
 		if (rottenfood(otmp)) {
